@@ -24,9 +24,11 @@
 #include <glib.h>
 #include "folder.h"
 
+extern gboolean manual_update;
+
 FolderClass *vcal_folder_get_class();
 void vcal_folder_gtk_init(void);
 void vcal_folder_gtk_done(void);
 GSList *vcal_folder_get_waiting_events(void);
-
+GSList *vcal_folder_get_webcal_events(void);
 #endif
