@@ -20,12 +20,12 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#  include "pluginconfig.h"
 #endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <glib.h>
-#include <glib/gi18n.h>
+#include "gettext.h"
 #include <ical.h>
 #include "utils.h"
 #include "vcalendar.h"
@@ -87,26 +87,32 @@ static GSList *answer_find(VCalEvent *event, Answer *answer)
 
 gchar *vcal_manager_answer_get_text(enum icalparameter_partstat ans) 
 {
-	static gchar *replies[5]={"accepted", "tentatively accepted", "declined", "did not answer",
-				  "unknown"};
+	static gchar *replies[5]={
+		N_("accepted"),
+		N_("tentatively accepted"),
+		N_("declined"),
+		N_("did not answer"),
+		N_("unknown")
+	};
+
 	switch (ans) {
 	case ICAL_PARTSTAT_ACCEPTED:
-		return replies[0];
+		return _(replies[0]);
 		break;
 	case ICAL_PARTSTAT_DECLINED:
-		return replies[2];
+		return _(replies[2]);
 		break;
 	case ICAL_PARTSTAT_TENTATIVE:
-		return replies[1];
+		return _(replies[1]);
 		break;
 	case ICAL_PARTSTAT_NEEDSACTION:
-		return replies[3];
+		return _(replies[3]);
 	case ICAL_PARTSTAT_DELEGATED:
 	case ICAL_PARTSTAT_COMPLETED:
 	case ICAL_PARTSTAT_X:
 	case ICAL_PARTSTAT_INPROCESS:
 	case ICAL_PARTSTAT_NONE:
-		return replies[4];
+		return _(replies[4]);
 		break;			
 	}
 	return NULL;
@@ -114,22 +120,28 @@ gchar *vcal_manager_answer_get_text(enum icalparameter_partstat ans)
 
 gchar *vcal_manager_cutype_get_text(enum icalparameter_cutype type) 
 {
-	static gchar *replies[5]={"individual", "group", "resource", "room",
-				  "unknown"};
+	static gchar *replies[5]={
+		N_("individual"),
+		N_("group"),
+		N_("resource"),
+		N_("room"),
+		N_("unknown")
+	};
+
 	switch (type) {
 	case ICAL_CUTYPE_INDIVIDUAL:
-		return replies[0];
+		return _(replies[0]);
 		break;
 	case ICAL_CUTYPE_GROUP:
-		return replies[1];
+		return _(replies[1]);
 		break;
 	case ICAL_CUTYPE_RESOURCE:
-		return replies[2];
+		return _(replies[2]);
 		break;
 	case ICAL_CUTYPE_ROOM:
-		return replies[3];
+		return _(replies[3]);
 	default:
-		return replies[4];
+		return _(replies[4]);
 		break;			
 	}
 	return NULL;
@@ -233,6 +245,7 @@ void vcal_manager_event_print(VCalEvent *event)
 		"event->end\t\t%s\n"
 		"event->summary\t\t%s\n"
 		"event->description\t%s\n"
+		"event->url\t%s\n"
 		"event->dtstart\t\t%s\n"
 		"event->dtend\t\t%s\n"
 		"event->tzid\t\t%s\n"
@@ -244,6 +257,7 @@ void vcal_manager_event_print(VCalEvent *event)
 		event->end,
 		event->summary,
 		event->description,
+		event->url,
 		event->dtstart,
 		event->dtend,
 		event->tzid,
@@ -271,18 +285,26 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 	gchar *attendee  = NULL;
 	gchar *body, *headers, *qpbody;
 	gchar **lines = NULL;
-	gchar *tmpfile = g_strdup_printf("%s%cevt-%d-%s", g_get_tmp_dir(),
-				      G_DIR_SEPARATOR, getuid(), event->uid);
+	gchar *tmpfile = NULL;
 	gchar *tmpstr = NULL;
-	icalcomponent *calendar, *ievent;
+	icalcomponent *calendar, *ievent, *timezone, *tzc;
+	struct icalperiodtype rtime;
 	icalproperty *attprop;
 	icalproperty *orgprop;
 	icalparameter *param = NULL;
 	enum icalparameter_partstat status = ICAL_PARTSTAT_NEEDSACTION;
 	int i = 0;
+	gchar *sanitized_uid = g_strdup(event->uid);
+	
+	subst_for_filename(sanitized_uid);
 
+	tmpfile = g_strdup_printf("%s%cevt-%d-%s", g_get_tmp_dir(),
+				      G_DIR_SEPARATOR, getuid(), sanitized_uid);
+	g_free(sanitized_uid);
 	if (!account) {
 		g_free(organizer);
+		g_free(tmpfile);
+		debug_print("no account found\n");
 		return NULL;
 	}
 
@@ -320,31 +342,72 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 		param = icalparameter_new_cn(tmpstr);
 		icalproperty_add_parameter(orgprop, param);
 		g_free(tmpstr); tmpstr = NULL;
+	} else if (event->orgname && strlen(event->orgname)) {
+		param = icalparameter_new_cn(event->orgname);
+		icalproperty_add_parameter(orgprop, param);
 	}
 	
+	timezone = icalcomponent_new(ICAL_VTIMEZONE_COMPONENT);
+	
+	icalcomponent_add_property(timezone,
+		icalproperty_new_tzid("GMT"));
+	
+	tzc = icalcomponent_new(ICAL_XSTANDARD_COMPONENT);
+	icalcomponent_add_property(tzc,
+		icalproperty_new_dtstart(
+			icaltime_from_string("19700101T000000")));
+	icalcomponent_add_property(tzc,
+		icalproperty_new_tzoffsetfrom(0.0));
+	icalcomponent_add_property(tzc,
+		icalproperty_new_tzoffsetto(0.0));
+	icalcomponent_add_property(tzc,
+		icalproperty_new_tzname("Greenwich meridian time"));
+
+        icalcomponent_add_component(timezone, tzc);
+
 	ievent = 
 	    icalcomponent_vanew(
                 ICAL_VEVENT_COMPONENT,
                 icalproperty_new_uid(event->uid),
+		icalproperty_vanew_dtstamp(icaltime_from_timet(time(NULL), TRUE),
+			icalparameter_new_tzid("GMT"), 0),
 		icalproperty_vanew_dtstart((icaltime_from_string(event->dtstart)),
-			icalparameter_new_tzid(tzname[1]), 0),
+			icalparameter_new_tzid("GMT"), 0),
 		icalproperty_vanew_dtend((icaltime_from_string(event->dtend)),
-			icalparameter_new_tzid(tzname[1]), 0),
+			icalparameter_new_tzid("GMT"), 0),
 		icalproperty_new_description(event->description),
 		icalproperty_new_summary(event->summary),
 		icalproperty_new_sequence(event->sequence + 1),
 		icalproperty_new_class("PUBLIC"),
 		icalproperty_new_transp("OPAQUE"),
+		icalproperty_new_location(""),
+		icalproperty_new_status(ICAL_STATUS_CONFIRMED),
+		icalproperty_vanew_created(icaltime_from_timet(time(NULL), TRUE),
+			icalparameter_new_tzid("GMT"), 0),
+		icalproperty_vanew_lastmodified(icaltime_from_timet(time(NULL), TRUE),
+			icalparameter_new_tzid("GMT"), 0),
+		
                 orgprop,
                 0
                 );
 
-	if (!event) {
+	if (!ievent) {
 		g_warning ("can't generate event");
+		g_free(organizer);
+		g_free(tmpfile);
 		return NULL;
 	}
+	
+	if (!icalcomponent_get_first_component(calendar, ICAL_VTIMEZONE_COMPONENT))
+	        icalcomponent_add_component(calendar, timezone);
+
         icalcomponent_add_component(calendar, ievent);
 
+	if (event->url && strlen(event->url)) {
+		attprop = icalproperty_new_url(event->url);
+        	icalcomponent_add_property(ievent, attprop);
+	}
+
 	if (is_reply) {
 		/* dump only this attendee */
 		attprop =
@@ -370,6 +433,12 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 		GSList *cur = event->answers;
 		while (cur && cur->data) {
 			Answer *a = (Answer *)cur->data;
+
+			if (a->cutype == 0)
+				a->cutype = ICAL_CUTYPE_INDIVIDUAL;
+			if (a->answer == 0)
+				a->answer = ICAL_PARTSTAT_NEEDSACTION;
+
 			attprop =
 	                	icalproperty_vanew_attendee(
                 		    a->attendee,
@@ -402,6 +471,107 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 		return NULL;
 	}
 
+	body = g_strdup_printf("%s"
+			       "\n"
+			       "%s", headers, icalcomponent_as_ical_string(calendar));
+	
+	str_write_to_file(body, tmpfile); 
+	chmod(tmpfile, S_IRUSR|S_IWUSR);
+
+	g_free(body);
+	g_free(headers);
+	icalcomponent_free(calendar);
+	g_free(attendee);
+	g_free(organizer);
+	
+	return tmpfile;	
+}
+
+static gchar *write_headers_ical(PrefsAccount 	*account, 
+			    icalcomponent	*ievent,
+			    gchar		*orga);
+
+gchar *vcal_manager_icalevent_dump(icalcomponent *event, gchar *orga, icalcomponent *use_calendar)
+{
+	PrefsAccount *account = cur_account;
+	gchar *attendee  = NULL;
+	gchar *body, *headers, *qpbody;
+	gchar **lines = NULL;
+	gchar *tmpfile = NULL;
+	gchar *tmpstr = NULL;
+	icalcomponent *calendar;
+	icalproperty *attprop, *prop;
+	icalproperty *orgprop;
+	icalparameter *param = NULL;
+	icalcomponent *ievent = NULL;
+	
+	enum icalparameter_partstat status = ICAL_PARTSTAT_NONE;
+	int i = 0;
+
+	ievent = icalcomponent_new_clone(event);
+
+	prop = icalcomponent_get_first_property(ievent, ICAL_UID_PROPERTY);
+	if (prop) {
+		gchar *sanitized_uid = g_strdup(icalproperty_get_uid(prop));
+
+		subst_for_filename(sanitized_uid);
+
+		tmpfile = g_strdup_printf("%s%cevt-%d-%s", g_get_tmp_dir(),
+					      G_DIR_SEPARATOR, getuid(), sanitized_uid);
+		g_free(sanitized_uid);
+		icalproperty_free(prop);
+	} else {
+		tmpfile = g_strdup_printf("%s%cevt-%d-%p", g_get_tmp_dir(),
+				      G_DIR_SEPARATOR, getuid(), ievent);
+	}
+
+	if (!account) {
+		g_free(tmpfile);
+		icalcomponent_free(ievent);
+		return NULL;
+	}
+
+	attendee = g_strdup_printf("MAILTO:%s", account->address);
+	
+	tzset();
+	
+	if (use_calendar != NULL) {
+		calendar = use_calendar;
+		g_free(tmpfile);
+		tmpfile = NULL;
+	} else
+		calendar = 
+        		icalcomponent_vanew(
+        		    ICAL_VCALENDAR_COMPONENT,
+	        	    icalproperty_new_version("2.0"),
+        		    icalproperty_new_prodid(
+                		 "-//Sylpheed-Claws//NONSGML Sylpheed-Claws Calendar//EN"),
+			    icalproperty_new_calscale("GREGORIAN"),
+			    icalproperty_new_method(ICAL_METHOD_PUBLISH),
+        		    0
+	        	    ); 	
+
+	if (!calendar) {
+		g_warning ("can't generate calendar");
+		g_free(tmpfile);
+		icalcomponent_free(ievent);
+		return NULL;
+	}
+	
+        icalcomponent_add_component(calendar, ievent);
+
+	if (use_calendar)
+		return NULL;
+
+	headers = write_headers_ical(account, ievent, orga);
+
+	if (!headers) {
+		g_warning("can't get headers");
+		g_free(tmpfile);
+		icalcomponent_free(calendar);
+		return NULL;
+	}
+
 	lines = g_strsplit(icalcomponent_as_ical_string(calendar), "\n", 0);
 	qpbody = g_strdup("");
 	
@@ -430,32 +600,35 @@ gchar *vcal_manager_event_dump(VCalEvent *event, gboolean is_reply, gboolean is_
 	g_free(headers);
 	icalcomponent_free(calendar);
 	g_free(attendee);
-	g_free(organizer);
 	
 	return tmpfile;	
 }
 
 VCalEvent * vcal_manager_new_event	(const gchar 	*uid, 
 					 const gchar	*organizer,
+					 const gchar	*orgname,
 					 const gchar	*summary,
 					 const gchar	*description,
 					 const gchar	*dtstart,
 					 const gchar	*dtend,
 					 const gchar	*tzid,
+					 const gchar	*url,
 					 enum icalproperty_method method,
-					 gint 		 sequence)
+					 gint 		 sequence,
+					 enum icalproperty_kind type)
 {
 	VCalEvent *event = g_new0(VCalEvent, 1);
 	
 	event->uid 		= g_strdup(uid?uid:"");
 	event->organizer 	= g_strdup(organizer?organizer:"");
+	event->orgname	 	= g_strdup(orgname?orgname:"");
 
-	if (dtend) {
+	if (dtend && strlen(dtend)) {
 		time_t tmp = icaltime_as_timet((icaltime_from_string(dtend)));
 		event->end	= g_strdup(ctime(&tmp));
 	}
 	
-	if (dtstart) {
+	if (dtstart && strlen(dtstart)) {
 		time_t tmp = icaltime_as_timet((icaltime_from_string(dtstart)));
 		time_t tmp_utc = icaltime_as_timet((icaltime_from_string(dtstart)));
 		event->start	= g_strdup(ctime(&tmp));
@@ -464,14 +637,18 @@ VCalEvent * vcal_manager_new_event	(const gchar 	*uid,
 	event->dtend		= g_strdup(dtend?dtend:"");
 	event->summary		= g_strdup(summary?summary:"");
 	event->description	= g_strdup(description?description:"");
+	event->url		= g_strdup(url?url:"");
 	event->tzid		= g_strdup(tzid?tzid:"");
 	event->method		= method;
 	event->sequence		= sequence;
+	event->type		= type;
 	return event;
 }
 					 
 void vcal_manager_free_event (VCalEvent *event)
 {
+	if (!event)
+		return;
 	g_free(event->uid);
 	g_free(event->organizer);
 	g_free(event->start);
@@ -481,6 +658,7 @@ void vcal_manager_free_event (VCalEvent *event)
 	g_free(event->dtend);
 	g_free(event->tzid);
 	g_free(event->description);
+	g_free(event->url);
 	g_free(event);
 }
 
@@ -496,8 +674,14 @@ gchar *vcal_manager_get_event_path(void)
 
 gchar *vcal_manager_get_event_file(const gchar *uid)
 {
-	return g_strconcat(vcal_manager_get_event_path(), G_DIR_SEPARATOR_S,
-			   uid, NULL);
+	gchar *tmp = g_strdup(uid);
+	gchar *res = NULL;
+
+	subst_for_filename(tmp);
+	res = g_strconcat(vcal_manager_get_event_path(), G_DIR_SEPARATOR_S,
+			   tmp, NULL);
+	g_free(tmp);
+	return res;
 }
 
 PrefsAccount *vcal_manager_get_account_from_event(VCalEvent *event)
@@ -531,8 +715,10 @@ void vcal_manager_save_event (VCalEvent *event)
 	
 	tag = xml_tag_new("event");
 	xml_tag_add_attr(tag, xml_attr_new("organizer", event->organizer));
+	xml_tag_add_attr(tag, xml_attr_new("orgname", event->orgname));
 	xml_tag_add_attr(tag, xml_attr_new("summary", event->summary));
 	xml_tag_add_attr(tag, xml_attr_new("description", event->description));
+	xml_tag_add_attr(tag, xml_attr_new("url", event->url));
 	xml_tag_add_attr(tag, xml_attr_new("dtstart", event->dtstart));
 	xml_tag_add_attr(tag, xml_attr_new("dtend", event->dtend));
 	xml_tag_add_attr(tag, xml_attr_new("tzid", event->tzid));
@@ -544,10 +730,19 @@ void vcal_manager_save_event (VCalEvent *event)
 	tmp = g_strdup_printf("%d", tmp_method);
 	xml_tag_add_attr(tag, xml_attr_new("method", tmp));
 	g_free(tmp);
+
 	tmp = g_strdup_printf("%d", event->sequence);
 	xml_tag_add_attr(tag, xml_attr_new("sequence", tmp));
 	g_free(tmp);
 	
+	tmp = g_strdup_printf("%d", event->type);
+	xml_tag_add_attr(tag, xml_attr_new("type", tmp));
+	g_free(tmp);
+	
+	tmp = g_strdup_printf("%lu", event->postponed);
+	xml_tag_add_attr(tag, xml_attr_new("postponed", tmp));
+	g_free(tmp);
+	
 	xmlnode = xml_node_new(tag, NULL);
 	rootnode = g_node_new(xmlnode);
 	
@@ -590,13 +785,12 @@ void vcal_manager_save_event (VCalEvent *event)
 		return;
 	}
  
-	if (vcalprefs.export_enable) {
-		if (vcal_meeting_export_calendar(vcalprefs.export_path)) {
-			if (vcalprefs.export_command &&
-			    strlen(vcalprefs.export_command))
-				execute_command_line(
-					vcalprefs.export_command, TRUE);
-		}
+	if (vcal_meeting_export_calendar(vcalprefs.export_path)) {
+		if (vcalprefs.export_enable &&
+		    vcalprefs.export_command &&
+		    strlen(vcalprefs.export_command))
+			execute_command_line(
+				vcalprefs.export_command, TRUE);
 	}
 }
 
@@ -604,11 +798,13 @@ static VCalEvent *event_get_from_xml (const gchar *uid, GNode *node)
 {
 	XMLNode *xmlnode;
 	GList *list;
-	gchar *org = NULL, *summary = NULL;
-	gchar *dtstart = NULL, *dtend = NULL, *tzid = NULL, *description = NULL;
+	gchar *org = NULL, *summary = NULL, *orgname = NULL;
+	gchar *dtstart = NULL, *dtend = NULL, *tzid = NULL, *description = NULL, *url = NULL;
 	VCalEvent *event = NULL;
 	enum icalproperty_method method = ICAL_METHOD_REQUEST;
+	enum icalproperty_kind type = ICAL_VEVENT_COMPONENT;
 	gint sequence = 0;
+	time_t postponed = (time_t)0;
 	
 	g_return_val_if_fail(node->data != NULL, NULL);
 
@@ -625,28 +821,40 @@ static VCalEvent *event_get_from_xml (const gchar *uid, GNode *node)
 		if (!attr || !attr->name || !attr->value) continue;
 		if (!strcmp(attr->name, "organizer"))
 			org = g_strdup(attr->value);
+		if (!strcmp(attr->name, "orgname"))
+			orgname = g_strdup(attr->value);
 		if (!strcmp(attr->name, "summary"))
 			summary = g_strdup(attr->value);
 		if (!strcmp(attr->name, "description"))
 			description = g_strdup(attr->value);
+		if (!strcmp(attr->name, "url"))
+			url = g_strdup(attr->value);
 		if (!strcmp(attr->name, "dtstart"))
 			dtstart = g_strdup(attr->value);
 		if (!strcmp(attr->name, "dtend"))
 			dtend = g_strdup(attr->value);
 		if (!strcmp(attr->name, "tzid"))
 			tzid = g_strdup(attr->value);
+		if (!strcmp(attr->name, "type"))
+			type = atoi(attr->value);
 		if (!strcmp(attr->name, "method"))
 			method = atoi(attr->value);
 		if (!strcmp(attr->name, "sequence"))
 			sequence = atoi(attr->value);
+		if (!strcmp(attr->name, "postponed"))
+			postponed = atoi(attr->value);
 	}
 
-	event = vcal_manager_new_event(uid, org, summary, description,
-					dtstart, dtend, tzid, method, sequence);
-	
+	event = vcal_manager_new_event(uid, org, orgname, summary, description, 
+					dtstart, dtend, tzid, url, method, 
+					sequence, type);
+
+	event->postponed = postponed;
+
 	g_free(org); 
 	g_free(summary); 
 	g_free(description); 
+	g_free(url); 
 	g_free(dtstart); 
 	g_free(dtend);
 	g_free(tzid);
@@ -680,6 +888,7 @@ static VCalEvent *event_get_from_xml (const gchar *uid, GNode *node)
 
 		event->answers = g_slist_append(event->answers, answer_new(attendee, name, answer, cutype));
 		g_free(attendee);
+		g_free(name);
 		node = node->next;
 	}
 	
@@ -776,7 +985,8 @@ static gchar *write_headers(PrefsAccount 	*account,
 	gchar *queue_headers = NULL;
 	gchar *method_str = NULL;
 	gchar *attendees = NULL;
-	
+	enum icalparameter_partstat status;
+	gchar *prefix = NULL;
 	memset(subject, 0, sizeof(subject));
 	memset(date, 0, sizeof(date));
 	
@@ -834,32 +1044,117 @@ static gchar *write_headers(PrefsAccount 	*account,
 		queue_headers = g_strdup("");
 	}
 	
-	if (is_reply) 
+	prefix = "";
+	if (is_reply) {
 		method_str = "REPLY";
-	else if (event->method == ICAL_METHOD_PUBLISH)
+		status = vcal_manager_get_reply_for_attendee(event, account->address);
+		if (status == ICAL_PARTSTAT_ACCEPTED)
+			prefix = _("Accepted: ");
+		else if (status == ICAL_PARTSTAT_DECLINED)
+			prefix = _("Declined: ");
+		else if (status == ICAL_PARTSTAT_TENTATIVE)
+			prefix = _("Tentatively Accepted: ");
+		else 
+			prefix = "Re: ";
+	} else if (event->method == ICAL_METHOD_PUBLISH)
 		method_str = "PUBLISH";
 	else
 		method_str = "REQUEST";		
 					
 	result = g_strdup_printf("%s"
-				"From: <%s>\n"
+				"From: %s <%s>\n"
 				"To: <%s>\n"
 				"Subject: %s%s\n"
 				"Date: %s\n"
 				"MIME-Version: 1.0\n"
 				"Content-Type: text/calendar; method=%s; charset=\"%s\"\n"
-				"Content-Transfer-Encoding: quoted-printable\n",
+				"Content-Transfer-Encoding: 8bit\n",
 				queue_headers,
+				is_reply ? account->name:(event->orgname?event->orgname:""),
 				is_reply ? account->address:event->organizer,
-				is_reply ? event->organizer:attendees,
-				is_reply ? "Re: ":"",
+				is_reply ? event->organizer:(attendees?attendees:event->organizer),
+				prefix,
+				subject,
+				date,
+				method_str,
+				CS_UTF_8);
+				
+	g_free(save_folder);
+	g_free(queue_headers);
+
+	return result;			
+                                                                               
+
+}
+
+static gchar *write_headers_ical(PrefsAccount 	*account, 
+			    icalcomponent	*ievent,
+			    gchar 		*orga)
+{
+	gchar subject[512];
+	gchar date[128];
+	gchar *result = NULL;
+	gchar *method_str = NULL;
+	gchar *attendees = NULL;
+	gchar *summary = NULL;
+	gchar *organizer = NULL;
+	gchar *orgname = NULL;
+	icalproperty *prop = NULL;
+	time_t t;
+
+	memset(subject, 0, sizeof(subject));
+	memset(date, 0, sizeof(date));
+	
+	prop = icalcomponent_get_first_property(ievent, ICAL_SUMMARY_PROPERTY);
+	if (prop) {
+		summary = g_strdup(icalproperty_get_summary(prop));
+		icalproperty_free(prop);
+	} else {
+		summary = g_strdup("");
+	}
+
+	prop = icalcomponent_get_first_property(ievent, ICAL_ORGANIZER_PROPERTY);
+	if (prop) {
+		organizer = g_strdup(icalproperty_get_organizer(prop));
+		if (icalproperty_get_parameter_as_string(prop, "CN") != NULL)
+			orgname = g_strdup(icalproperty_get_parameter_as_string(prop, "CN"));
+
+		icalproperty_free(prop);
+	} else {
+		organizer = orga? g_strdup(orga):g_strdup("");
+	}
+
+	prop = icalcomponent_get_first_property(ievent, ICAL_DTSTART_PROPERTY);
+	if (prop) {
+		t = icaltime_as_timet(icalproperty_get_dtstart(prop));
+		get_rfc822_date_from_time_t(date, sizeof(date), t);
+	} else {
+		get_rfc822_date(date, sizeof(date));
+	}
+
+	conv_encode_header(subject, 511, summary, strlen(summary), FALSE);
+			
+	method_str = "PUBLISH";
+					
+	result = g_strdup_printf("From: %s <%s>\n"
+				"To: <%s>\n"
+				"Subject: %s%s\n"
+				"Date: %s\n"
+				"MIME-Version: 1.0\n"
+				"Content-Type: text/calendar; method=%s; charset=\"%s\"; vcalsave=\"no\"\n"
+				"Content-Transfer-Encoding: quoted-printable\n",
+				orgname?orgname:"",
+				!strncmp(organizer, "MAILTO:", 7) ? organizer+7 : organizer,
+				account->address,
+				"",
 				subject,
 				date,
 				method_str,
 				conv_get_outgoing_charset_str());
-				
-	g_free(save_folder);
-	g_free(queue_headers);
+	
+	g_free(orgname);
+	g_free(organizer);
+	g_free(summary);
 
 	return result;			
                                                                                
