@@ -20,11 +20,11 @@
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
+#  include "pluginconfig.h"
 #endif
 
 #include <ical.h>
-#include <glib.h>
-#include <glib/gi18n.h>
+#include "gettext.h"
 #include <gtk/gtk.h>
 
 #include "mimeview.h"
@@ -34,6 +34,7 @@
 #include "vcal_folder.h"
 #include "vcal_meeting_gtk.h"
 #include "prefs_account.h"
+#include "prefs_common.h"
 #include "account.h"
 #include "codeconv.h"
 #include "xml.h"
@@ -43,6 +44,14 @@
 
 MimeViewerFactory vcal_viewer_factory;
 
+static void refresh_folder_contents(VCalViewer *vcalviewer);
+static GdkColor uri_color = {
+	(gulong)0,
+	(gushort)0,
+	(gushort)0,
+	(gushort)0
+};
+
 struct _VCalViewer
 {
 	MimeViewer mimeviewer;
@@ -61,10 +70,12 @@ struct _VCalViewer
 	GtkWidget *end;
 	GtkWidget *summary;
 	GtkWidget *description;
+	gchar	  *url;
 	GtkWidget *answer;
 	GtkWidget *button;
 	GtkWidget *reedit;
 	GtkWidget *cancel;
+	GtkWidget *uribtn;
 	GtkWidget *attendees;
 };
 
@@ -110,7 +121,6 @@ static void vcal_viewer_clear_viewer(MimeViewer *_mimeviewer)
 		g_free(vcalviewer->tmpfile);
 		vcalviewer->tmpfile = NULL;
 	}
-	gtk_widget_hide(vcalviewer->scrolledwin);
 	vcalviewer->mimeinfo = NULL;
 }
 
@@ -134,6 +144,16 @@ static gchar *get_email_from_attendee_property(icalproperty *p)
 	return email;
 }
 
+static gchar *get_name_from_organizer_property(icalproperty *p)
+{
+	gchar *tmp = NULL;
+	
+	if (p && icalproperty_get_parameter_as_string(p, "CN") != NULL)
+		tmp = g_strdup(icalproperty_get_parameter_as_string(p, "CN"));
+
+	return tmp;
+}
+
 static gchar *get_email_from_organizer_property(icalproperty *p)
 {
 	gchar *tmp = NULL;
@@ -144,7 +164,7 @@ static gchar *get_email_from_organizer_property(icalproperty *p)
 
 	if (!tmp) 
 		return NULL;
-		
+
 	if (!strncasecmp(tmp, "MAILTO:", strlen("MAILTO:")))
 		email = g_strdup(tmp+strlen("MAILTO:"));
 	else
@@ -232,15 +252,24 @@ static icalcomponent *vcalviewer_get_component(const gchar *file)
 	return comp;
 }
 
+#define GTK_LABEL_SET_TEXT_TRIMMED(label, text) {		\
+	gchar *tmplbl = strretchomp(g_strdup(text));		\
+	gtk_label_set_text(label, tmplbl);			\
+	g_free(tmplbl);						\
+}
+
 static void vcalviewer_reset(VCalViewer *vcalviewer) 
 {
-	gtk_label_set_text(GTK_LABEL(vcalviewer->type), "-");
-	gtk_label_set_text(GTK_LABEL(vcalviewer->who), "-");
-	gtk_label_set_text(GTK_LABEL(vcalviewer->summary), "-");
-	gtk_label_set_text(GTK_LABEL(vcalviewer->description), "-");
-	gtk_label_set_text(GTK_LABEL(vcalviewer->start), "-");
-	gtk_label_set_text(GTK_LABEL(vcalviewer->end), "-");
-	gtk_label_set_text(GTK_LABEL(vcalviewer->attendees), "-");
+	GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), "-");
+	GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->who), "-");
+	GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->summary), "-");
+	GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->description), "-");
+	GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->start), "-");
+	GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->end), "-");
+	GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->attendees), "-");
+	g_free(vcalviewer->url);
+	vcalviewer->url = NULL;
+	gtk_widget_hide(vcalviewer->uribtn);
 }
 
 static gchar *get_attendee_replying(VCalViewer *vcalviewer)
@@ -409,7 +438,7 @@ static void vcalviewer_answer_set_choices(VCalViewer *vcalviewer, VCalEvent *eve
 		gtk_widget_show(vcalviewer->answer);
 		gtk_widget_show(vcalviewer->button);
 	} else {
-		gtk_combo_box_append_text(GTK_COMBO_BOX(vcalviewer->answer), _("-"));
+		gtk_combo_box_append_text(GTK_COMBO_BOX(vcalviewer->answer), "-");
 		gtk_widget_set_sensitive(vcalviewer->answer, FALSE);
 		gtk_widget_set_sensitive(vcalviewer->button, FALSE);
 	}
@@ -437,23 +466,30 @@ void vcalviewer_display_event (VCalViewer *vcalviewer, VCalEvent *event)
 	gchar *attendees = NULL;
 	gboolean firstatt = TRUE;
 	gboolean mine = FALSE;
+	gchar *label = NULL;
 	
 	if (!event)
 		return;
 	if (!vcalviewer)
 		return;
 
-	if (event->method == ICAL_METHOD_REQUEST) {
+	if (event->type == ICAL_VTODO_COMPONENT) {
+		label = g_strjoin(" ", _("You have a Todo item."),
+				_("Details follow:"), NULL);
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
+	} else if (event->method == ICAL_METHOD_REQUEST) {
 		if (account_find_from_address(event->organizer)) {
-			gtk_label_set_text(GTK_LABEL(vcalviewer->type), 
-				_("You have created a meeting. Details follow:"));
+			label = g_strjoin(" ", _("You have created a meeting."),
+					_("Details follow:"), NULL);
+			GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
 			mine = TRUE;
-		} else 
-			gtk_label_set_text(GTK_LABEL(vcalviewer->type), 
-				_("You have been invited to a meeting. Details follow:"));
+		} else {
+			label = g_strjoin(" ", _("You have been invited to a meeting."),
+						_("Details follow:"), NULL);
+			GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
+		}
 	}
 	else if (event->method == ICAL_METHOD_REPLY) {
-		gchar *label = NULL;
 		enum icalparameter_partstat answer = get_attendee_reply(vcalviewer);
 		enum icalparameter_cutype cutype = get_attendee_replying_cutype(vcalviewer);
 		gchar *attendee = get_attendee_replying(vcalviewer);
@@ -462,55 +498,69 @@ void vcalviewer_display_event (VCalViewer *vcalviewer, VCalEvent *event)
 		vcal_manager_update_answer(event, attendee, name, answer, cutype);
 
 		if (!attendee) {
-			label = g_strdup_printf(
-				_("You have received an answer to an unknown meeting proposal. Details follow:"));
+			label = g_strjoin(" ",
+				_("You have received an answer to an unknown meeting proposal."),
+				_("Details follow:"), NULL);
+			GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
 		} else {
 			label = g_strdup_printf(_("You have received an answer to a meeting proposal.\n"
 				"%s has %s the invitation whose details follow:"),
 				attendee, vcal_manager_get_reply_text_for_attendee(event, attendee));
-			gtk_label_set_text(GTK_LABEL(vcalviewer->type), label);
+			GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
 			g_free(attendee);
 		}
-			
-		g_free(label);
 	} else if (event->method == ICAL_METHOD_CANCEL) {
-		Folder *folder = NULL;
-		gtk_label_set_text(GTK_LABEL(vcalviewer->type), 
-			_("A meeting to which you had been invited has been cancelled. Details follow:"));
+		label = g_strjoin(" ",
+				_("A meeting to which you had been invited has been cancelled."),
+				_("Details follow:"), NULL);
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
 		vcal_manager_save_event(event);
-		folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
-		if (folder)
-			folder_item_scan(folder->inbox);
-	} else 
-		gtk_label_set_text(GTK_LABEL(vcalviewer->type), 
-			_("You have been forwarded an appointment. Details follow:"));
+		refresh_folder_contents(vcalviewer);
+	} else {
+		label = g_strjoin(" ", _("You have been forwarded an appointment."),
+				_("Details follow:"), NULL);
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
+	}
+	g_free(label);
 
-	
-	if (event->organizer && strlen(event->organizer)) {
-		gtk_label_set_text(GTK_LABEL(vcalviewer->who), event->organizer);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->who), "-");
-	
+	if (event->orgname && strlen(event->orgname)) {
+		gchar *addr = g_strconcat(event->orgname, " <", event->organizer, ">", NULL);
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->who), addr);
+		g_free(addr);
+	} else if (event->organizer && strlen(event->organizer)) {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->who), event->organizer);
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->who), "-");
+	}
 	if (event->summary && strlen(event->summary)) {
-		gtk_label_set_text(GTK_LABEL(vcalviewer->summary), event->summary);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->summary), "-");
-	
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->summary), event->summary);
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->summary), "-");
+	}
 	if (event->description && strlen(event->description)) {
-		gtk_label_set_text(GTK_LABEL(vcalviewer->description), event->description);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->description), "-");
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->description), event->description);
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->description), "-");
+	}
+	g_free(vcalviewer->url);
+	if (event->url && strlen(event->url)) {
+		vcalviewer->url = g_strdup(event->url);
+		gtk_widget_show(vcalviewer->uribtn);
+	} else {
+		vcalviewer->url = NULL;
+		gtk_widget_hide(vcalviewer->uribtn);
+	}
 	
 	if (event->start && strlen(event->start)) {
-		gtk_label_set_text(GTK_LABEL(vcalviewer->start), event->start);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->start), "-");
-	
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->start), event->start);
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->start), "-");
+	}
 	if (event->end && strlen(event->end)) {
-		gtk_label_set_text(GTK_LABEL(vcalviewer->end), event->end);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->end), "-");
-	
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->end), event->end);
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->end), "-");
+	}
 	for (list = vcal_manager_get_answers_emails(event); 
 	     list && list->data; list = list->next) {
 	     	gchar *attendee = (gchar *)list->data;
@@ -537,13 +587,16 @@ void vcalviewer_display_event (VCalViewer *vcalviewer, VCalEvent *event)
 		g_free(name);
 	}
 
-	if (attendees && strlen(attendees))
-		gtk_label_set_text(GTK_LABEL(vcalviewer->attendees), attendees);
-	else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->attendees), "-");
-	
+	if (attendees && strlen(attendees)) {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->attendees), attendees);
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->attendees), "-");
+	}
 	if (!mine)
-		vcalviewer_answer_set_choices(vcalviewer, event, event->method);
+		if (event->type != ICAL_VTODO_COMPONENT)
+			vcalviewer_answer_set_choices(vcalviewer, event, event->method);
+		else
+			vcalviewer_answer_set_choices(vcalviewer, event, ICAL_METHOD_PUBLISH);
 	else {
 		vcalviewer_answer_set_choices(vcalviewer, event, ICAL_METHOD_REPLY);
 		gtk_widget_show(vcalviewer->reedit);
@@ -596,15 +649,16 @@ gchar *vcalviewer_get_uid_from_mimeinfo(MimeInfo *mimeinfo)
 
 }
 
-static void vcalviewer_get_request_values(VCalViewer *vcalviewer, MimeInfo *mimeinfo) 
+static void vcalviewer_get_request_values(VCalViewer *vcalviewer, MimeInfo *mimeinfo, gboolean is_todo) 
 {
 	icalproperty *iprop = NULL;
-	gchar *org = NULL, *summary = NULL, *description = NULL;
-	gchar *dtstart = NULL, *dtend = NULL, *tzid = NULL;
+	gchar *org = NULL, *summary = NULL, *description = NULL, *url = NULL;
+	gchar *dtstart = NULL, *dtend = NULL, *tzid = NULL, *orgname = NULL;
 	enum icalproperty_method method = ICAL_METHOD_REQUEST;
 	VCalEvent *event = NULL;
 	gchar *tmp;
 	const gchar *charset = procmime_mimeinfo_get_parameter(mimeinfo, "charset");
+	const gchar *saveme =  procmime_mimeinfo_get_parameter(mimeinfo, "vcalsave");
 	gchar * uid = vcalviewer_get_uid_from_mimeinfo(mimeinfo);
 	gint sequence = 0;
 	
@@ -632,27 +686,48 @@ static void vcalviewer_get_request_values(VCalViewer *vcalviewer, MimeInfo *mime
 		vcal_manager_free_event(event);
 		g_free(uid);
 		return;
+	} else if (event) {
+		vcal_manager_free_event(event);
 	}
-	
+
 	/* load it and register it */
 
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_ORGANIZER_PROPERTY);
 	if (iprop) {
 		tmp = get_email_from_organizer_property(iprop);
-		org = conv_codeset_strdup(tmp, charset, CS_UTF_8);
+		if (!g_utf8_validate(tmp, -1, NULL))
+			org = conv_codeset_strdup(tmp, charset, CS_UTF_8);
+		else
+			org = g_strdup(tmp);
 		g_free(tmp);
 		icalproperty_free(iprop);
+		orgname = get_name_from_organizer_property(iprop);
 	} 
 	
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_SUMMARY_PROPERTY);
 	if (iprop) {
-		summary = conv_codeset_strdup(icalproperty_get_summary(iprop), charset, CS_UTF_8);
+		if (!g_utf8_validate(icalproperty_get_summary(iprop), -1, NULL))
+			summary = conv_codeset_strdup(icalproperty_get_summary(iprop), charset, CS_UTF_8);
+		else
+			summary = g_strdup(icalproperty_get_summary(iprop));
 		icalproperty_free(iprop);
 	} 
 
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_DESCRIPTION_PROPERTY);
 	if (iprop) {
-		description = conv_codeset_strdup(icalproperty_get_description(iprop), charset, CS_UTF_8);
+		if (!g_utf8_validate(icalproperty_get_description(iprop), -1, NULL))
+			description = conv_codeset_strdup(icalproperty_get_description(iprop), charset, CS_UTF_8);
+		else
+			description = g_strdup(icalproperty_get_description(iprop));
+		icalproperty_free(iprop);
+	} 
+	
+	iprop = vcalviewer_get_property(vcalviewer, ICAL_URL_PROPERTY);
+	if (iprop) {
+		if (!g_utf8_validate(icalproperty_get_url(iprop), -1, NULL))
+			url = conv_codeset_strdup(icalproperty_get_url(iprop), charset, CS_UTF_8);
+		else
+			url = g_strdup(icalproperty_get_url(iprop));
 		icalproperty_free(iprop);
 	} 
 	
@@ -683,14 +758,18 @@ static void vcalviewer_get_request_values(VCalViewer *vcalviewer, MimeInfo *mime
 	}
 	
 	event = vcal_manager_new_event( uid,
-					org, summary, description,
-					dtstart, dtend, tzid, method, sequence);
+					org, orgname, summary, description, 
+					dtstart, dtend, tzid, url, method, sequence, 
+					is_todo?ICAL_VTODO_COMPONENT:ICAL_VEVENT_COMPONENT);
 	vcalviewer_get_attendees(vcalviewer, event);
-	vcal_manager_save_event(event);
+	if (!saveme || strcmp(saveme,"no"))
+		vcal_manager_save_event(event);
 
 	g_free(org); 
+	g_free(orgname); 
 	g_free(summary);
 	g_free(description);
+	g_free(url);
 	g_free(uid);
 	vcalviewer_display_event(vcalviewer, event);
 	vcal_manager_free_event(event);
@@ -701,6 +780,12 @@ static void vcalviewer_get_reply_values(VCalViewer *vcalviewer, MimeInfo *mimein
 	icalproperty *iprop = NULL;
 	gchar *tmp = NULL;
 	const gchar *charset = procmime_mimeinfo_get_parameter(mimeinfo, "charset");
+	gchar *label = NULL;
+	enum icalparameter_partstat answer = get_attendee_reply(vcalviewer);
+	enum icalparameter_cutype cutype = get_attendee_replying_cutype(vcalviewer);
+	gchar *attendee = get_attendee_replying(vcalviewer);
+	gchar *name = get_attendee_replying_name(vcalviewer);
+
 	if (!charset)
 		charset=CS_ISO_8859_1;
 
@@ -716,58 +801,113 @@ static void vcalviewer_get_reply_values(VCalViewer *vcalviewer, MimeInfo *mimein
 		} 
 	} 
 	
-	/* unknown answer */
-	gtk_label_set_text(GTK_LABEL(vcalviewer->type), 
-		_("You have received an answer to an unknown meeting proposal. Details follow:"));
+	/* unknown answer but get the replier anyway */
+		
+	if (!attendee) {
+		label = g_strjoin(" ",
+			_("You have received an answer to an unknown meeting proposal."),
+			_("Details follow:"), NULL);
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
+	} else {
+		label = g_strdup_printf(_("You have received an answer to a meeting proposal.\n"
+			"%s has %s the invitation whose details follow:"),
+			attendee, vcal_manager_answer_get_text(answer));
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->type), label);
+		g_free(attendee);
+	}
+
+	g_free(label);
+
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_ORGANIZER_PROPERTY);
 	if (iprop) {
-		gchar *org;
+		gchar *org, *orgname;
 		tmp = get_email_from_organizer_property(iprop);
-		org = conv_codeset_strdup(tmp, charset, CS_UTF_8);
+		if (!g_utf8_validate(tmp, -1, NULL))
+			org = conv_codeset_strdup(tmp, charset, CS_UTF_8);
+		else 
+			org = g_strdup(tmp);
 		g_free(tmp);
-		gtk_label_set_text(GTK_LABEL(vcalviewer->who), org);
+		tmp = get_name_from_organizer_property(iprop);
+		printf(" %s !\n", tmp);
+		if (tmp && !g_utf8_validate(tmp, -1, NULL))
+			orgname = conv_codeset_strdup(tmp, charset, CS_UTF_8);
+		else if (tmp)
+			orgname = g_strdup(tmp);
+		g_free(tmp);
+		if (orgname) {
+			gchar *addr = g_strconcat(orgname, " <", org, ">", NULL);
+			GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->who), addr);
+			g_free(addr);
+		} else {
+			GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->who), org);
+		}
 		icalproperty_free(iprop);
 		g_free(org);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->who), "-");
-
+		g_free(orgname);
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->who), "-");
+	}
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_SUMMARY_PROPERTY);
 	if (iprop) {
-		tmp = conv_codeset_strdup(icalproperty_get_summary(iprop), charset, CS_UTF_8);
-		gtk_label_set_text(GTK_LABEL(vcalviewer->summary), tmp);
+		if (!g_utf8_validate(icalproperty_get_summary(iprop), -1, NULL))
+			tmp = conv_codeset_strdup(icalproperty_get_summary(iprop), charset, CS_UTF_8);
+		else
+			tmp = g_strdup(icalproperty_get_summary(iprop));
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->summary), tmp);
 		g_free(tmp);
 		icalproperty_free(iprop);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->summary), "-");
-
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->summary), "-");
+	}
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_DESCRIPTION_PROPERTY);
 	if (iprop) {
-		tmp = conv_codeset_strdup(icalproperty_get_description(iprop), charset, CS_UTF_8);
-		gtk_label_set_text(GTK_LABEL(vcalviewer->description), tmp);
+		if (!g_utf8_validate(icalproperty_get_description(iprop), -1, NULL))
+			tmp = conv_codeset_strdup(icalproperty_get_description(iprop), charset, CS_UTF_8);
+		else
+			tmp = g_strdup(icalproperty_get_description(iprop));
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->description), tmp);
 		g_free(tmp);
 		icalproperty_free(iprop);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->description), "-");
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->description), "-");
+	}
+	iprop = vcalviewer_get_property(vcalviewer, ICAL_URL_PROPERTY);
+	g_free(vcalviewer->url);
+	if (iprop) {
+		if (!g_utf8_validate(icalproperty_get_url(iprop), -1, NULL))
+			tmp = conv_codeset_strdup(icalproperty_get_url(iprop), charset, CS_UTF_8);
+		else
+			tmp = g_strdup(icalproperty_get_url(iprop));
+		vcalviewer->url = g_strdup(tmp);
+		g_free(tmp);
+		icalproperty_free(iprop);
+		gtk_widget_show(vcalviewer->uribtn);
+	} else {
+		vcalviewer->url = NULL;
+		gtk_widget_hide(vcalviewer->uribtn);
+	}
 
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_DTSTART_PROPERTY);
 	if (iprop) {
 		struct icaltimetype itt = (icalproperty_get_dtstart(iprop));
 		time_t tmp = icaltime_as_timet(itt);
-		gtk_label_set_text(GTK_LABEL(vcalviewer->start), ctime(&tmp));
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->start), ctime(&tmp));
 		icalproperty_free(iprop);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->start), "-");
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->start), "-");
+	}
 
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_DTEND_PROPERTY);
 	if (iprop) {
 		struct icaltimetype itt = (icalproperty_get_dtstart(iprop));
 		time_t tmp = icaltime_as_timet(itt);
-		gtk_label_set_text(GTK_LABEL(vcalviewer->end), ctime(&tmp));
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->end), ctime(&tmp));
 		icalproperty_free(iprop);
-	} else
-		gtk_label_set_text(GTK_LABEL(vcalviewer->end), "-");
-	
-	gtk_label_set_text(GTK_LABEL(vcalviewer->attendees), "-"); 
+	} else {
+		GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->end), "-");
+	}
+
+	GTK_LABEL_SET_TEXT_TRIMMED(GTK_LABEL(vcalviewer->attendees), "-"); 
 
 	vcalviewer_answer_set_choices(vcalviewer, NULL, ICAL_METHOD_REPLY);
 }
@@ -775,7 +915,7 @@ static void vcalviewer_get_reply_values(VCalViewer *vcalviewer, MimeInfo *mimein
 static void vcalviewer_get_info(VCalViewer *vcalviewer, MimeInfo *mimeinfo) 
 {
 	icalproperty *method = NULL;
-	
+	gboolean is_todo = FALSE;
 	gchar *tmpfile = get_tmpfile(vcalviewer);
 	
 	if (vcalviewer->comp) {
@@ -799,15 +939,20 @@ static void vcalviewer_get_info(VCalViewer *vcalviewer, MimeInfo *mimeinfo)
 	
 	method = vcalviewer_get_property(vcalviewer, ICAL_METHOD_PROPERTY);
 	
-	if (icalcomponent_isa(vcalviewer->comp) == ICAL_VCALENDAR_COMPONENT) {
+	if (vcalviewer->icomp && icalcomponent_isa(vcalviewer->icomp) == ICAL_VTODO_COMPONENT)
+		is_todo = TRUE;
+debug_print("is_todo %d\n", is_todo);
+	if (icalcomponent_isa(vcalviewer->comp) == ICAL_VCALENDAR_COMPONENT && method) {
 		if (icalproperty_get_method(method) == ICAL_METHOD_REQUEST
 		||  icalproperty_get_method(method) == ICAL_METHOD_PUBLISH
 		||  icalproperty_get_method(method) == ICAL_METHOD_CANCEL)
-			vcalviewer_get_request_values(vcalviewer, mimeinfo);
+			vcalviewer_get_request_values(vcalviewer, mimeinfo, is_todo);
 		else if (icalproperty_get_method(method) == ICAL_METHOD_REPLY)
 			vcalviewer_get_reply_values(vcalviewer, mimeinfo);
 		else
 			vcalviewer_reset(vcalviewer);
+	} else if (is_todo && !method) {
+		vcalviewer_get_request_values(vcalviewer, mimeinfo, is_todo);
 	} else
 		vcalviewer_reset(vcalviewer);
 }
@@ -818,7 +963,11 @@ static void vcal_viewer_show_mimepart(MimeViewer *_mimeviewer, const gchar *file
 {
 	VCalViewer *vcalviewer = (VCalViewer *) _mimeviewer;
 	s_vcalviewer = vcalviewer;
-	
+
+	if (mimeinfo == NULL) {
+		vcal_viewer_clear_viewer(_mimeviewer);
+		return;
+	}
 	debug_print("vcal_viewer_show_mimepart : %s\n", file);
 
 	vcal_viewer_clear_viewer(_mimeviewer);
@@ -827,6 +976,12 @@ static void vcal_viewer_show_mimepart(MimeViewer *_mimeviewer, const gchar *file
 	vcalviewer->file = g_strdup(file);
 	vcalviewer->mimeinfo = mimeinfo;
 	vcalviewer_get_info(vcalviewer, mimeinfo);
+	GTK_EVENTS_FLUSH();
+	gtk_widget_set_size_request(vcalviewer->description, 
+		vcalviewer->scrolledwin->allocation.width - 200, -1);
+	gtk_label_set_line_wrap(GTK_LABEL(vcalviewer->summary), TRUE);
+	gtk_label_set_line_wrap(GTK_LABEL(vcalviewer->description), TRUE);
+	gtk_label_set_line_wrap(GTK_LABEL(vcalviewer->attendees), FALSE);
 }
 
 void vcalviewer_reload(void)
@@ -843,6 +998,8 @@ static void vcal_viewer_destroy_viewer(MimeViewer *_mimeviewer)
 
 	debug_print("vcal_viewer_destroy_viewer\n");
 
+	if (s_vcalviewer == vcalviewer)
+		s_vcalviewer = NULL;
 	vcal_viewer_clear_viewer(_mimeviewer);
 	g_free(vcalviewer);
 }
@@ -852,7 +1009,8 @@ static gboolean vcalviewer_reedit_cb(GtkButton *widget, gpointer data)
 	VCalViewer *vcalviewer = (VCalViewer *)data;
 	gchar * uid = vcalviewer_get_uid_from_mimeinfo(vcalviewer->mimeinfo);
 	VCalEvent *event = NULL;
-
+	
+	s_vcalviewer = vcalviewer;
 	/* see if we have it registered and more recent */
 	event = vcal_manager_load_event(uid);
 	vcal_meeting_create(event);
@@ -860,6 +1018,27 @@ static gboolean vcalviewer_reedit_cb(GtkButton *widget, gpointer data)
 	return TRUE;
 }
 
+static gboolean vcalviewer_uribtn_cb(GtkButton *widget, gpointer data)
+{
+	VCalViewer *vcalviewer = (VCalViewer *)data;
+
+	if (vcalviewer->url)
+		open_uri(vcalviewer->url, prefs_common.uri_cmd);
+
+	return TRUE;
+}
+
+static void refresh_folder_contents(VCalViewer *vcalviewer)
+{
+	Folder *folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
+	if (folder) {
+		MainWindow *mainwin = mainwindow_get_mainwindow();
+		folder_item_scan(folder->inbox);
+		folderview_check_new(folder);
+		if (mainwin->summaryview->folder_item == folder->inbox)
+			summary_show(mainwin->summaryview, folder->inbox);
+	}
+}
 static gboolean vcalviewer_cancel_cb(GtkButton *widget, gpointer data)
 {
 	VCalViewer *vcalviewer = (VCalViewer *)data;
@@ -867,34 +1046,41 @@ static gboolean vcalviewer_cancel_cb(GtkButton *widget, gpointer data)
 	VCalEvent *event = NULL;
 	VCalMeeting *meet = NULL;
 	gchar *file = NULL;
-	Folder *folder;
-	
-	gint val = alertpanel_full(_("Cancel meeting"),
+	gint val = 0;
+	val = alertpanel_full(_("Cancel meeting"),
 				   _("Are you sure you want to cancel this meeting?\n"
 			           "A notification will be sent to attendees."),
-				   GTK_STOCK_YES, GTK_STOCK_NO, NULL, FALSE,
-				   NULL, ALERT_WARNING, G_ALERTALTERNATE);
+				   GTK_STOCK_NO, GTK_STOCK_YES, NULL, FALSE,
+				   NULL, ALERT_WARNING, G_ALERTDEFAULT);
 
-	if (val != G_ALERTDEFAULT)
+	if (val != G_ALERTALTERNATE)
 		return TRUE;
-
+	
+	s_vcalviewer = vcalviewer;
+	
 	event = vcal_manager_load_event(uid);
 	if (!event)
 		return TRUE;
 	event->method = ICAL_METHOD_CANCEL;
 	
 	meet = vcal_meeting_create_hidden(event);
-	vcal_meeting_send(meet);
+	if (!vcal_meeting_send(meet)) {
+		event->method = ICAL_METHOD_REQUEST;
+		vcal_manager_save_event(event);
+		vcal_manager_free_event(event);
+		refresh_folder_contents(vcalviewer);
+		vcalviewer_reset(vcalviewer);
+		return TRUE;
+	}
+
 	vcal_manager_save_event(event);
 	
 	file = vcal_manager_get_event_file(event->uid);
 	unlink(file);
 	vcal_manager_free_event(event);
 	g_free(file);
-	folder = folder_find_from_name ("vCalendar", vcal_folder_get_class());
-	if (folder)
-		folder_item_scan(folder->inbox);
-	vcalviewer_reload();
+	refresh_folder_contents(vcalviewer);
+	vcalviewer_reset(vcalviewer);
 
 	return TRUE;
 }
@@ -914,6 +1100,8 @@ static gboolean vcalviewer_action_cb(GtkButton *widget, gpointer data)
 		return TRUE;
 	}
 	
+	s_vcalviewer = vcalviewer;
+
 	iprop = vcalviewer_get_property(vcalviewer, ICAL_UID_PROPERTY);
 	if (iprop) {
 		event = vcal_manager_load_event(icalproperty_get_uid(iprop));
@@ -944,8 +1132,10 @@ static gboolean vcalviewer_action_cb(GtkButton *widget, gpointer data)
 };
 
 #define TABLE_ADD_LINE(label_text, widget) { 				\
-	GtkWidget *label = gtk_label_new(_("<span weight=\"bold\">" 	\
-						label_text "</span>")); \
+	gchar *tmpstr = g_strdup_printf("<span weight=\"bold\">%s</span>",\
+				label_text);				\
+	GtkWidget *label = gtk_label_new(tmpstr);		 	\
+	g_free(tmpstr);							\
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);		\
 	gtk_misc_set_alignment (GTK_MISC(label), 1, 0);			\
 	gtk_table_attach (GTK_TABLE (vcalviewer->table), 		\
@@ -967,7 +1157,8 @@ MimeViewer *vcal_viewer_create(void)
 	VCalViewer *vcalviewer;
 	int i = 0;
 	GtkWidget *hbox = NULL;
-	
+	GtkStyle *style;
+
 	debug_print("Creating vcal view...\n");
 	vcalviewer = g_new0(VCalViewer, 1);
 	vcalviewer->mimeviewer.factory = &vcal_viewer_factory;
@@ -986,16 +1177,43 @@ MimeViewer *vcal_viewer_create(void)
 	vcalviewer->description = gtk_label_new("description");
 	vcalviewer->attendees = gtk_label_new("attendees");
 	vcalviewer->answer = gtk_combo_box_new_text();
+	vcalviewer->url = NULL;
 	vcalviewer->button = gtk_button_new_with_label(_("Answer"));
 	vcalviewer->reedit = gtk_button_new_with_label(_("Edit meeting..."));
 	vcalviewer->cancel = gtk_button_new_with_label(_("Cancel meeting..."));
+	vcalviewer->uribtn = gtk_button_new_with_label(_("Launch website"));
 	vcalviewer_answer_set_choices(vcalviewer, NULL, ICAL_METHOD_REQUEST);
 	hbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(hbox), vcalviewer->answer, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), vcalviewer->button, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), vcalviewer->reedit, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), vcalviewer->cancel, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), vcalviewer->uribtn, FALSE, FALSE, 0);
 	
+	gtk_label_set_selectable(GTK_LABEL(vcalviewer->type), TRUE);
+	gtk_label_set_selectable(GTK_LABEL(vcalviewer->who), TRUE);
+	gtk_label_set_selectable(GTK_LABEL(vcalviewer->start), TRUE);
+	gtk_label_set_selectable(GTK_LABEL(vcalviewer->end), TRUE);
+	gtk_label_set_selectable(GTK_LABEL(vcalviewer->summary), TRUE);
+	gtk_label_set_selectable(GTK_LABEL(vcalviewer->description), TRUE);
+	gtk_label_set_selectable(GTK_LABEL(vcalviewer->attendees), TRUE);
+	
+/*	gtk_widget_ensure_style(vcalviewer->who);
+	style = gtk_style_copy
+		(gtk_widget_get_style(vcalviewer->who));
+	style->fg[GTK_STATE_NORMAL]   = uri_color;
+	style->fg[GTK_STATE_ACTIVE]   = uri_color;
+	style->fg[GTK_STATE_PRELIGHT] = uri_color;
+	gtk_widget_set_style(vcalviewer->who, style);
+
+	gtk_widget_ensure_style(vcalviewer->attendees);
+	style = gtk_style_copy
+		(gtk_widget_get_style(vcalviewer->attendees));
+	style->fg[GTK_STATE_NORMAL]   = uri_color;
+	style->fg[GTK_STATE_ACTIVE]   = uri_color;
+	style->fg[GTK_STATE_PRELIGHT] = uri_color;
+	gtk_widget_set_style(vcalviewer->attendees, style);
+*/
 	g_signal_connect(G_OBJECT(vcalviewer->button), "clicked",
 			 G_CALLBACK(vcalviewer_action_cb), vcalviewer);
 
@@ -1005,14 +1223,18 @@ MimeViewer *vcal_viewer_create(void)
 	g_signal_connect(G_OBJECT(vcalviewer->cancel), "clicked",
 			 G_CALLBACK(vcalviewer_cancel_cb), vcalviewer);
 
-	TABLE_ADD_LINE("Event:", vcalviewer->type);
-	TABLE_ADD_LINE("Organizer:", vcalviewer->who);
-	TABLE_ADD_LINE("Summary:", vcalviewer->summary);
-	TABLE_ADD_LINE("Starting:", vcalviewer->start);
-	TABLE_ADD_LINE("Ending:", vcalviewer->end);
-	TABLE_ADD_LINE("Description:", vcalviewer->description);
-	TABLE_ADD_LINE("Attendees:", vcalviewer->attendees);
-	TABLE_ADD_LINE("Action:", hbox);
+	g_signal_connect(G_OBJECT(vcalviewer->uribtn), "clicked",
+			 G_CALLBACK(vcalviewer_uribtn_cb), vcalviewer);
+
+	TABLE_ADD_LINE(_("Event:"), vcalviewer->type);
+	TABLE_ADD_LINE(_("Organizer:"), vcalviewer->who);
+	TABLE_ADD_LINE(_("Summary:"), vcalviewer->summary);
+	TABLE_ADD_LINE(_("Starting:"), vcalviewer->start);
+	TABLE_ADD_LINE(_("Ending:"), vcalviewer->end);
+	TABLE_ADD_LINE(_("Description:"), vcalviewer->description);
+	TABLE_ADD_LINE(_("Attendees:"), vcalviewer->attendees);
+	gtk_label_set_line_wrap(GTK_LABEL(vcalviewer->attendees), FALSE);
+	TABLE_ADD_LINE(_("Action:"), hbox);
 	
 	vcalviewer->scrolledwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_add_with_viewport(
@@ -1023,7 +1245,6 @@ MimeViewer *vcal_viewer_create(void)
 				       GTK_POLICY_AUTOMATIC);
 	
 	gtk_widget_show_all(vcalviewer->scrolledwin);
-	
 	return (MimeViewer *) vcalviewer;
 }
 
@@ -1038,6 +1259,20 @@ MimeViewerFactory vcal_viewer_factory =
 };
 
 static gint alert_timeout_tag = 0;
+static gint scan_timeout_tag = 0;
+
+static gint vcal_webcal_check(gpointer data)
+{
+	Folder *root = folder_find_from_name ("vCalendar", vcal_folder_get_class());
+
+	if (prefs_common.work_offline)
+		return TRUE;
+	
+	manual_update = FALSE;
+	folderview_check_new(root);
+	manual_update = TRUE;
+	return TRUE;
+}
 
 void vcalendar_init(void)
 {
@@ -1053,7 +1288,7 @@ void vcalendar_init(void)
 		folder = folder_new(vcal_folder_get_class(), "vCalendar", NULL);
 		folder->klass->create_tree(folder);
 		folder_add(folder);
-		folder_scan_tree(folder);
+		folder_scan_tree(folder, TRUE);
 	}
 	if (folder)
 		folder_item_scan(folder->inbox);
@@ -1064,14 +1299,38 @@ void vcalendar_init(void)
 	alert_timeout_tag = gtk_timeout_add(60*1000, 
 				(GtkFunction)vcal_meeting_alert_check, 
 				(gpointer)NULL);
+	scan_timeout_tag = gtk_timeout_add(3600*1000, 
+				(GtkFunction)vcal_webcal_check, 
+				(gpointer)NULL);
+	if (prefs_common.enable_color) {
+		gtkut_convert_int_to_gdk_color(prefs_common.uri_col,
+				       &uri_color);
+	}
 }
 
 void vcalendar_done(void)
 {
+	MainWindow *mainwin = mainwindow_get_mainwindow();
+	FolderView *folderview = NULL;
+	FolderItem *fitem = NULL;
+
+	if (mainwin == NULL)
+		return;
+	folderview = mainwin->folderview;
+	fitem = folderview->summaryview->folder_item;
+
+	if (fitem && 
+	    fitem->folder->klass == vcal_folder_get_class()) {
+		folderview_unselect(folderview);
+		summary_clear_all(folderview->summaryview);
+	}
+
 	mimeview_unregister_viewer_factory(&vcal_viewer_factory);
 	folder_unregister_class(vcal_folder_get_class());
 	vcal_folder_gtk_done();
 	vcal_prefs_done();
 	gtk_timeout_remove(alert_timeout_tag);
 	alert_timeout_tag = 0;
+	gtk_timeout_remove(scan_timeout_tag);
+	scan_timeout_tag = 0;
 }
